#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <netdb.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

/* Fonctionnel (res en 64 bits pour éviter les débordements au moment de res*base) */
uint32_t squaring_exponentiation(uint32_t base, uint32_t exposant, uint32_t mod){
  uint64_t res = 1;
  while(exposant !=  0){
    if(exposant % 2 == 1){
      res = (res * base) % mod;
    }
    exposant = exposant >> 1;
    base = (base * base) % mod;
  }
  return (uint32_t)res;
} 

/* Fonctionnel */
int miller_rabin(uint32_t n,uint32_t a) {
   int res = 0;
   uint32_t n1 = n - 1;
   
   uint32_t d = n1 >> 1;
   
   int j = 1;
   
   while ((d & 1) == 0) d >>= 1, ++j;
   
    uint64_t t = a, p = a;
   
    while (d >>= 1) { //square and multiply: a^d mod n
      p = p*p % n;
      if (d & 1) t = t*p % n;
    }
    if (t == 1 || t == n1){
       res = 1;
    }

   for (int k=1 ; k<j ; ++k) {
      t = t*t % n;
      if (t == n1) res = 1;
      if (t <= 1) break;
   }
   return res; 
}

// Donne l'inverse de a modulo mod (mod premier) d'apres euclide etendue
uint64_t inverseModulaire(uint32_t a, uint32_t mod){
  uint32_t r = mod; int64_t v = 0;     // r = mod*u - a*v   (ici on se fout de u)
  uint32_t rp = a; int64_t vp = 1;  // rp = mod*up - a*up
  uint32_t q; // Quotient
  uint32_t rtemp; int64_t vtemp;  // Var de stockage

  while(rp != 1){
    q = r / rp;
    // On stock pour les calculs
    rtemp = r;
    vtemp = v;
    // On decale
    r = rp;
    v = vp;
    // On calcule le reste et le nouveau coeff vp
    rp = rtemp % rp;
    vp = vtemp - (q * vp);
  }

  return vp < 0 ? mod + vp : vp;
}

uint32_t randomN(int8_t nbBits){
  if(nbBits > 32) { 
      nbBits = 32; 
  }
  uint32_t n = rand() % 2;
  for(int i = 0; i < nbBits - 1; i++) {
    n = n << 1;
    n += rand() % 2;
  }
  return n;
}

// fonction permettant de lire un fichier et de recuperer son contenu en char*
char* read_file(char* nameFile,int* size){
  // nameFile : nom du fichier en chaine de caractere
    int nbCarac= 1;
    char* text = calloc(nbCarac,sizeof(char));
    FILE* file = NULL;
    file = fopen(nameFile,"r");
     int actuelCarac = 0;

    if (file != NULL)
   {
       // Boucle de lecture des caractères un à un
       do
       {

           actuelCarac = fgetc(file); // On lit le caractère
           if(actuelCarac != EOF){
             text[nbCarac-1] = actuelCarac;
             nbCarac++;
             text = realloc(text, nbCarac*sizeof(char));
           }


       } while (actuelCarac != EOF); // On continue tant que fgetc n'a pas retourné EOF (fin de fichier)

       fclose(file);

   }
        *size = nbCarac-1;
       return text;
}